@@ -14,7 +14,7 @@ module SimpleSMT
 
     -- ** S-Expressions
   , SExpr(..)
-  , showsSExpr, readSExpr
+  , showsSExpr, ppSExpr, readSExpr
 
     -- ** Logging and Debugging
   , Logger(..)
@@ -100,6 +100,8 @@ module SimpleSMT
   , mod
   , divisible
   , realDiv
+  , toInt
+  , toReal
 
     -- ** Bit Vectors
   , concat
@@ -130,7 +132,7 @@ module SimpleSMT
 import Prelude hiding (not, and, or, abs, div, mod, concat, const)
 import qualified Prelude as P
 import Data.Char(isSpace)
-import Data.List(unfoldr)
+import Data.List(unfoldr,intersperse)
 import Data.Bits(testBit)
 import Data.IORef(newIORef, atomicModifyIORef, modifyIORef', readIORef,
                   writeIORef)
@@ -142,7 +144,7 @@ import Control.Concurrent(forkIO)
 import Control.Monad(forever,when)
 import Text.Read(readMaybe)
 import Data.Ratio((%), numerator, denominator)
-import Numeric(showHex, readHex)
+import Numeric(showHex, readHex, showFFloat)
 
 
 -- | Results of checking for satisfiability.
@@ -173,6 +175,38 @@ showsSExpr ex =
                 foldr (\e m -> showsSExpr e . showChar ' ' . m)
                 (showChar ')') es
 
+
+-- | Show an S-expression in a somewhat readbale fashion.
+ppSExpr :: SExpr -> ShowS
+ppSExpr = go 0
+  where
+  tab n = showString (replicate n ' ')
+  many  = foldr (.) id
+
+  new n e = showChar '\n' . tab n . go n e
+
+  small n es =
+    case es of
+      [] -> Just []
+      e : more
+        | n <= 0 -> Nothing
+        | otherwise -> case e of
+                         Atom x -> (showString x :) <$> small (n-1) more
+                         _      -> Nothing
+
+  go :: Int -> SExpr -> ShowS
+  go n ex =
+    case ex of
+      Atom x        -> showString x
+      List es
+        | Just fs <- small 5 es ->
+          showChar '(' . many (intersperse (showChar ' ') fs) . showChar ')'
+
+      List (Atom x : es) -> showString "(" . showString x .
+                                many (map (new (n+3)) es) . showString ")"
+
+      List es -> showString "(" . many (map (new (n+2)) es) . showString ")"
+
 -- | Parse an s-expression.
 readSExpr :: String -> Maybe (SExpr, String)
 readSExpr (c : more) | isSpace c = readSExpr more
@@ -578,11 +612,14 @@ bool b = const (if b then "true" else "false")
 -- | Integer literals.
 int :: Integer -> SExpr
 int x | x < 0     = neg (int (negate x))
-         | otherwise = Atom (show x)
+      | otherwise = Atom (show x)
 
 -- | Real (well, rational) literals.
 real :: Rational -> SExpr
-real x = realDiv (int (denominator x)) (int (numerator x))
+real x
+  | toRational y == x = Atom (showFFloat Nothing y "")
+  | otherwise = realDiv (int (numerator x)) (int (denominator x))
+  where y = fromRational x :: Double
 
 -- | A bit vector represented in binary.
 --
@@ -757,6 +794,14 @@ signExtend i x = List [ fam "sign_extend" [i], x ]
 zeroExtend :: Integer -> SExpr -> SExpr
 zeroExtend i x = List [ fam "zero_extend" [i], x ]
 
+-- | Satisfies @toInt x <= x@ (i.e., this is like Haskell's 'floor')
+toInt :: SExpr -> SExpr
+toInt e = fun "to_int" [e]
+
+-- | Promote an integer to a real
+toReal :: SExpr -> SExpr
+toReal e = fun "to_real" [e]
+
 -- | Extract a sub-sequence of a bit vector.
 extract :: SExpr -> Integer -> Integer -> SExpr
 extract x y z = List [ fam "extract" [y,z], x ]
